#!/usr/bin/env python3

"""
Created on 4 Nov 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)

source repo: scs_philips_hue

DESCRIPTION
The chroma utility is used to map environmental data domain values to chromaticity locations. Input data is received
from stdin, and is interpreted as a float value. The mapped value is written to stdout in the form of a JSON
scs_philips_hue.data.light.LightState document.

The chroma utility requires the chroma_conf.json document, specifying the parameters for the mapping.

SYNOPSIS
chroma.py [-v]

EXAMPLES
./aws_mqtt_subscriber.py -vce | ./node.py -c | ./chroma.py -v

FILES
~/SCS/hue/chroma_conf_set.json

DOCUMENT EXAMPLE - OUTPUT
{"PM10": {"bri": 254, "transitiontime": 90, "xy": [0.3521, 0.6198]}}

SEE ALSO
scs_philips_hue/chroma_conf
scs_philips_hue/desk

RESOURCES
https://en.wikipedia.org/wiki/Chromaticity
https://developers.meethue.com/documentation/core-concepts
"""

import json
import sys

from scs_core.data.json import JSONify

from scs_core.sys.logging import Logging
from scs_core.sys.signalled_exit import SignalledExit

from scs_host.sys.host import Host

from scs_philips_hue.cmd.cmd_chroma import CmdChroma

from scs_philips_hue.config.chroma_conf import ChromaConfSet

from scs_philips_hue.data.light.light_state import LightState


# --------------------------------------------------------------------------------------------------------------------

if __name__ == '__main__':

    # ----------------------------------------------------------------------------------------------------------------
    # cmd...

    cmd = CmdChroma()

    Logging.config('chroma', verbose=cmd.verbose)
    logger = Logging.getLogger()

    logger.info(cmd)

    try:
        # ------------------------------------------------------------------------------------------------------------
        # resources...

        # ChromaConf...
        chroma_confs = ChromaConfSet.load(Host)

        if chroma_confs is None:
            logger.error("ChromaConfSet not available.")
            exit(1)

        logger.info(chroma_confs)

        # ChromaMapping...
        mappings = {}
        for name, chroma_conf in chroma_confs.confs.items():
            path = chroma_conf.path()

            if path is None:
                logger.error("%s: ChromaPath not available." % name)
                exit(1)

            logger.info("%s: %s" % (name, path))

            mappings[name] = chroma_conf.mapping(path)


        # ------------------------------------------------------------------------------------------------------------
        # run...

        # signal handler...
        SignalledExit.construct("chroma", cmd.verbose)

        # read stdin...
        for line in sys.stdin:
            datum = json.loads(line)

            if datum is None:
                break

            for name, chroma_conf in chroma_confs.confs.items():
                if name not in datum:
                    continue

                try:
                    value = float(datum[name])
                except (TypeError, ValueError) as ex:
                    logger.error("%s: %s" % (ex.__class__.__name__, datum))
                    continue

                logger.info("%s: domain value: %s" % (name, value))

                # interpolate...
                interpolation = mappings[name].interpolate(value)
                state = LightState(bri=chroma_conf.brightness, xy=interpolation,
                                   transition_time=chroma_conf.transition_time)

                print(JSONify.dumps({name: state}))
                sys.stdout.flush()

                break


    # ----------------------------------------------------------------------------------------------------------------
    # end...

    except KeyboardInterrupt:
        print(file=sys.stderr)

    except SystemExit:
        pass

    finally:
        logger.info("finishing")



"""
Created on 30 Oct 2017

@author: Bruno Beloff (bruno.beloff@southcoastscience.com)
"""

from scs_philips_hue.data.bridge.response import Response
from scs_philips_hue.data.light.light import Light, LightListEntry
from scs_philips_hue.data.light.light_name import LightName
from scs_philips_hue.data.light.light_scan import LightScan

from scs_philips_hue.manager.manager import Manager


# --------------------------------------------------------------------------------------------------------------------

class LightManager(Manager):
    """
    classdocs
    """

    __REQUEST_TIMEOUT = 5               # seconds

    # ----------------------------------------------------------------------------------------------------------------

    @classmethod
    def construct_all_from_jdict(cls, jdict):
        if not jdict:
            return None

        return {bridge_name: cls.construct_from_jdict(lm_jdict)
                for bridge_name, lm_jdict in jdict.items()}


    @classmethod
    def construct_all(cls, bridge_manager_group):
        return {bridge_name: cls(bridge_manager.host, bridge_manager.username)
                for bridge_name, bridge_manager in bridge_manager_group.items() if bridge_manager is not None}


    # ----------------------------------------------------------------------------------------------------------------

    def __init__(self, host, username):
        """
        Constructor
        """
        super().__init__(host, username)


    # ----------------------------------------------------------------------------------------------------------------

    def find_index_for_uid(self, unique_id):
        lights = self.find_all()

        for entry in lights:
            if entry.light.unique_id == unique_id:
                return entry.index

        return None


    def find_indices_for_name(self, name):
        lights = self.find_all()

        indices = []

        for entry in lights:
            if entry.light.name == name:
                indices.append(entry.index)

        return indices


    def find(self, index):
        request_path = '/lights/' + str(index)

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.get(request_path)
        finally:
            self._rest_client.close()

        # response...
        return Light.construct_from_jdict(jdict)


    def find_new(self):
        request_path = '/lights/new'

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.get(request_path)
        finally:
            self._rest_client.close()

        # response...
        report = LightScan.construct_from_jdict(jdict)

        return report


    def find_all(self):
        request_path = '/lights'

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.get(request_path)
        finally:
            self._rest_client.close()

        # response...
        lights = []

        # TODO: handle error case

        for index, light_jdict in jdict.items():
            lights.append(LightListEntry.construct_from_jdict(index, light_jdict))

        return lights


    # ----------------------------------------------------------------------------------------------------------------

    def search(self, device=None):
        request_path = '/lights'
        payload = device.as_json() if device else {}

        # request...
        self._rest_client.connect(self._host, self._username)

        try:
            jdict = self._rest_client.post(request_path, payload)
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response


    # ----------------------------------------------------------------------------------------------------------------

    def rename(self, index, name):
        request_path = '/lights/' + str(index)
        payload = LightName(name)

        # request...
        self._rest_client.connect(self._host, self._username, timeout=self.__REQUEST_TIMEOUT)

        try:
            jdict = self._rest_client.put(request_path, payload.as_json())
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response


    def set_state(self, index, state):
        request_path = '/lights/' + str(index) + '/state'

        # request...
        self._rest_client.connect(self._host, self._username, timeout=self.__REQUEST_TIMEOUT)

        try:
            jdict = self._rest_client.put(request_path, state.as_json())
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response


    def delete(self, index):
        request_path = '/lights/' + str(index)

        # request...
        self._rest_client.connect(self._host, self._username, timeout=self.__REQUEST_TIMEOUT)

        try:
            jdict = self._rest_client.delete(request_path)
        finally:
            self._rest_client.close()

        # response...
        response = Response.construct_from_jdict(jdict)

        return response
